@@ -34,6 +34,8 @@ def hint_func_abs(placer, hints):
         adv = 0
     else:
         adv = 1
+    if placer.size[1-adv] == 0:
+        return hints
     return [hint/placer.size[1-adv] if hint is not None else None for hint in hints]
 
 
@@ -44,7 +46,7 @@ class NodeB:
     """
     type = "node"
     selectable = False
-    def __init__(self, name=None):
+    def __init__(self, name=None, text=None):
         if not hasattr(self, "name"):
             self.name = name
         if not hasattr(self, "_position"):
@@ -59,6 +61,8 @@ class NodeB:
         self.image_mode = None #Other options are: "fit", "fill"
         self.image_ratio = 1
         self.hidden = False
+        if text is not None:
+            self.text = text
 
     @property
     def position(self):
@@ -111,6 +115,15 @@ class NodeB:
             raise KeyError(f"Node not found: {key}")
         return v
 
+class TextNodeB(NodeB):
+    type="text"
+    def __init__(self, text, *, scale=32, name=None):
+        super().__init__()
+        self.name = name
+        self.text = text
+        self.scale = scale
+
+
 class ButtonB(NodeB):
     """
       Basic button. Callback will be called with button's name as parameter. **text** is displayed text
@@ -157,6 +170,18 @@ class CentererB(NodeB):
         if child is not None:
             self.add_child(child)
     
+    @property
+    def child(self):
+        return self.children[0]
+    
+    @child.setter
+    def child(self, val):
+        if self.children:
+            self.children[0] = val
+        else:
+            self.children.append(val)
+        self.recalc_size()
+
     def recalc_size(self):
         tpos = (self.position.x + self.sep_x, self.position.y + self.sep_y)
         tsize = (max(self.size.x - self.sep_x*2, 0), max(self.size.y - self.sep_y*2, 0))
@@ -204,6 +229,7 @@ class RCPlacerB(NodeB):
     def add_child(self, child, size_hint=None):
         self.children.append(child)
         self.size_hints.append(size_hint)
+        self.recalc_size()
 
     def recalc_size(self):
         n = len(self.children)
@@ -271,11 +297,23 @@ class TextInputB(NodeB):
     selectable = True
     def __init__(self, text="", name="default", callback=None):
         super().__init__()
-        self.text = text
+        self._text = text
         self.callback = callback
         self.cursor = 0
         self.textalign = "left"
         self.name = name
+
+    @property
+    def text(self):
+        return self._text
+    
+    @text.setter
+    def text(self, value):
+        self._text = value
+        if len(self._text) < self.cursor:
+            self.cursor = len(self._text)
+        if self.cursor < 0:
+            self.cursor = 0
         
     def handle_event(self, event: GUIEvent):
         if event.type == "key":
@@ -285,8 +323,8 @@ class TextInputB(NodeB):
         if event.type == "speckey":
             if event.key == "backspace":
                 if self.cursor > 0:
-                    self.text = self.text[:self.cursor-1] + self.text[self.cursor:]
                     self.cursor -= 1
+                    self.text = self.text[:self.cursor] + self.text[self.cursor+1:]
             elif event.key == "delete":
                 if self.cursor < len(self.text):
                     self.text = self.text[:self.cursor] + self.text[self.cursor+1:]
@@ -351,9 +389,10 @@ class ProgressBarB(NodeB):
       Progress bar. **self.fraction** is how full it is, with value from 0 to 1 inclusively.
     """
     type = "progressbar"
-    def __init__(self):
+    def __init__(self, name=None):
         super().__init__()
         self.fraction = 0
+        self.name = name
 
 
 class ScrollableListB(NodeB):
@@ -419,10 +458,11 @@ class ScrollableStringListB(ScrollableListB):
       **override_node_type** changes type of buttons to node, so that they are renderer like usual NodeB.
       Call `self.update_view()` after changing **self.full_list** to update
     """
-    def __init__(self, callback=None, shown_items=10, override_node_type=True):
+    def __init__(self, callback=None, shown_items=10, override_node_type=True, name=None):
         super().__init__(shown_items)
         self.callback = callback
         self._override_node_type = override_node_type
+        self.name = name
     
     def callback_adapter(self, num):
         if self.callback is not None:
