@@ -12,25 +12,24 @@ default_hint_conf = {
     glfw.CONTEXT_CREATION_API: glfw.NATIVE_CONTEXT_API,
     glfw.CLIENT_API: glfw.OPENGL_API,
     glfw.OPENGL_PROFILE: glfw.OPENGL_CORE_PROFILE,
-    glfw.RESIZABLE: True,
     glfw.OPENGL_FORWARD_COMPAT: True,
     glfw.SAMPLES: 4,
+    glfw.REFRESH_RATE: 60,
 }
 
 
 fallback_hint_conf = {
     glfw.CONTEXT_VERSION_MAJOR: 3,
-    glfw.CONTEXT_VERSION_MINOR: 1,
+    glfw.CONTEXT_VERSION_MINOR: 3,
     glfw.DOUBLEBUFFER: True,
     glfw.DEPTH_BITS: 8,
     glfw.CONTEXT_CREATION_API: glfw.NATIVE_CONTEXT_API,
     glfw.CLIENT_API: glfw.OPENGL_API,
-    glfw.RESIZABLE: True,
     glfw.SAMPLES: 1,
 }
 
 class Window:
-    def __init__(self, width=800, height=600, title="QLibs window", swap_interval=1, hint_conf=default_hint_conf):
+    def __init__(self, width=800, height=600, title="QLibs window", swap_interval=1, hint_conf=default_hint_conf, resizable=True, fullscreen=False):
         self.width = width
         self.height = height
         self.resize_callback = None
@@ -42,12 +41,23 @@ class Window:
         self.flip_mouse_y = True #flipped relativly to usual math-y respresentation
 
         glfw.init()
+        glfw.window_hint(glfw.RESIZABLE, resizable)
         for k, v in hint_conf.items():
             glfw.window_hint(k, v)
-        
+        monitor = None
+        if fullscreen:
+            monitor = glfw.get_primary_monitor()
+            mode = glfw.get_video_mode(monitor)
+            width = mode.size.width
+            height = mode.size.height
+            #width = 1920
+            #height = 1080
+            glfw.window_hint(glfw.RED_BITS, mode.bits.red)
+            glfw.window_hint(glfw.GREEN_BITS, mode.bits.green)
+            glfw.window_hint(glfw.BLUE_BITS, mode.bits.blue)
+            glfw.window_hint(glfw.REFRESH_RATE, mode.refresh_rate)
         try:
-            #raise glfw.GLFWError("test")
-            self.window = glfw.create_window(width, height, title, None, None)
+            self.window = glfw.create_window(width, height, title, monitor, None)
         except glfw.GLFWError:
             logger.warn("Provided config is unavailable, using fallback config")
             glfw.default_window_hints()
@@ -73,6 +83,9 @@ class Window:
         except:
             self.ctx = moderngl.create_context(libgl='libGL.so.1')
 
+    def make_context_current(self):
+        glfw.make_context_current(self.window)
+
     def _on_resize(self, win, width, height):
         self.width = width
         self.height = height
