@@ -89,7 +89,7 @@ class VecBase:
         return self.__floordiv__(other)
 
     def __pos__(self):
-        return self
+        return self.__class__(*self)
 
     def __neg__(self):
         return self.__class__(*map(lambda x: -x, self))
@@ -215,7 +215,7 @@ class Vec2(VecBase):
         return math.atan2(self.y, self.x)
     
     @angle.setter
-    def _(self, value):
+    def angle(self, value):
         l = self.len()
         self.x, self.y = math.cos(value)*l, math.sin(value)*l
     
